#!/bin/bash 
# QLogic System Information Utility
# Copyright (C) 2006 QLogic Corporation (www.qlogic.com)
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

#-------------------------------------------------------------------------------
# This script attempts to gather troubleshooting information on 
# a variety of Linux hosts.  Files that exist in one distribution of
# Linux may not exist in another distribution (SuSE vs Red Hat).
# Please ignore any errors reported about files not existing.
#
# --QLogic Technical Support
#------------------------------------------------------------------------------

REVISION=1.8
CurDate=`/bin/date +%m%d%y_%H%M%S`
QLOGICSUPPORT="support@qlogic.com"
QL_EXIT=0 # Make it 1 to activate sending mail

#####################################################################
# Display Help
#####################################################################

if [ $# -ne 0 ] ; then
	case $1 in 
		-h | --help | -? )
	/bin/echo "To run this script type $0"
	exit 0
esac
fi

#####################################################################
# Check for root
#####################################################################

if test $EUID -ne 0 ; then
	/bin/echo "Must be root to run this Script..."
	exit 0
fi

#####################################################################
#Check Path
#####################################################################

	#check for /usr/sbin
	ADD_PATH="\/usr\/sbin"
	echo $PATH | grep ":$ADD_PATH:" >& /dev/null ||
	echo $PATH | grep "^$ADD_PATH:" >& /dev/null ||
	echo $PATH | grep ":$ADD_PATH$"  >& /dev/null
	ret=$?
	if [ "$ret" != "0" ]; then
		export PATH=/usr/sbin:$PATH
	fi
	ADD_PATH="\/sbin"
	echo $PATH | grep ":$ADD_PATH:" >& /dev/null ||
	echo $PATH | grep "^$ADD_PATH:" >& /dev/null ||
	echo $PATH | grep ":$ADD_PATH$"  >& /dev/null
	ret=$?
	if [ "$ret" != "0" ]; then
		export PATH=/sbin:$PATH
	fi
 
#####################################################################
# Create temporary directory based on SR Number to store output
#####################################################################

/bin/echo ""
/bin/echo "Hello,"
/bin/echo "This utility will now attempt to gather system information for debug purpose"

/bin/echo "Removing the previous archives..."
rm -rf /tmp/QLogicDiag*
LOGDIR="/tmp/QLogicDiag-${CurDate}-${REVISION}"
CURRENTDIR=$PWD
mkdir -p $LOGDIR

#####################################################################
# Gather /proc data
#####################################################################

/bin/echo -n "Gathering /proc files... "
mkdir $LOGDIR/proc
PROC_FILES="/proc/modules \
/proc/pci \
/proc/interrupts \
/proc/cmdline \
/proc/cpuinfo \
/proc/partition \
/proc/meminfo \
/proc/slabinfo"

for file in $PROC_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/proc
   fi
done
# Make sure /proc/scsi directory exists before gathering data
if test -d /proc/scsi
then
   cd $LOGDIR/proc
   cp -R /proc/scsi $LOGDIR/proc
fi
/bin/echo

#####################################################################
# Gather /etc data
#####################################################################

/bin/echo -n "Gathering /etc files... "
mkdir $LOGDIR/etc
ETC_FILES="/etc/modules.* \
/etc/modprobe.* \
/etc/qla*.conf \
/etc/hba.conf \
/etc/sysconfig/kernel \
/etc/sysconfig/hwconf \
/etc/mtab \
/etc/fstab \
/etc/modules.d/module-rename"

for file in $ETC_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/etc
   fi
done

ls -alRF /etc/rc.d/ > $LOGDIR/etc/ls_etcrcd.out

#####################################################################
# Gather API Library LOG.
#####################################################################

/bin/echo
/bin/echo "Gathering API Library Log..."
mkdir $LOGDIR/apilib-log
APILIB_FILES="/usr/lib/libqlsdm.log"
for file in $APILIB_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/apilib-log
   fi
done

#####################################################################
# Gather module info
####################################################################

/bin/echo -n "Gathering module information..."
KERN_VER=`uname -r`
MAJ_VER=`/bin/echo $KERN_VER | head -c3`
if test $MAJ_VER = "2.4"
then
   EXT=".o"
fi
if test $MAJ_VER = "2.6"
then
   EXT=".ko"
fi

find /lib/modules/$KERN_VER/kernel/drivers -name qla\*$EXT -print \
  -exec modinfo {} \; -exec /bin/echo \; > $LOGDIR/modinfo.out 2>&1

ls -alRF /lib/modules/ > $LOGDIR/ls_libmodules.out

lsmod > $LOGDIR/lsmod.out

#####################################################################
# Gather misc info
#####################################################################

/bin/echo
/bin/echo -n "Gathering miscellaneous system info..."
mkdir $LOGDIR/misc
lspci -v              > $LOGDIR/misc/lspci.out 2>&1
df -lk                > $LOGDIR/misc/df.out 2>&1
fdisk -l              > $LOGDIR/misc/fdisk.out 2>&1
ps -ealf              > $LOGDIR/misc/ps.out
ls -lF /usr/bin/*gcc* > $LOGDIR/misc/gcc.out
gcc --version        >> $LOGDIR/misc/gcc.out
iptables --list       > $LOGDIR/misc/iptables.out
ifconfig              > $LOGDIR/misc/ifconfig.out
chkconfig --list      > $LOGDIR/misc/chkconfig.out
which lsscsi >& /dev/null 
if test $? -eq 0 ; 
	then 
	lsscsi --verbose > $LOGDIR/misc/lsscsi.out
fi
mount > $LOGDIR/misc/mount.out

#Full Reporting features
sysctl -a > $LOGDIR/misc/sysctl.out 2&>1
lsof > $LOGDIR/misc/lsof.out  2&>1

#move pro info to line 71
#interrupt information and kernel command line information

vmstat -a > $LOGDIR/misc/vmstat.out
free > $LOGDIR/misc/free.out

#####################################################################
# Gather SANsurfer and driver source info
#####################################################################

/bin/echo
/bin/echo -n "Gathering SANsurfer and driver source info... "
mkdir $LOGDIR/sms
if test -d /opt/QLogic*
then
   ls -alRF /opt/QLogic* > $LOGDIR/sms/ls_optQLogic.out
fi

ls -alRF /usr/local/bin > $LOGDIR/sms/ls_usrlocalbin.out

if test -d /usr/src
then
   ls -alRF /usr/src/ > $LOGDIR/sms/ls_usrsrc.out
fi

#Omitted for now, file can get too large
#if test -d /usr/src/qlogic
#then
#   tar -czf $LOGDIR/sms/usrsrcqlogic.tgz -C /usr/src/qlogic .
#   /bin/echo -n "QLogic driver source"
#fi
/bin/echo

#####################################################################
# Gather System Log info
#####################################################################

/bin/echo -n "Gathering syslog info..."
mkdir $LOGDIR/logs
LOG_FILES="/var/log/messages* \
/var/log/localmessages* \
/var/log/warn* \
/var/log/dmesg* \
/var/log/boot*"

for file in $LOG_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/logs
   fi
done

dmesg > $LOGDIR/logs/dmesg.out

#####################################################################
# Gather OS info
#####################################################################

/bin/echo
/bin/echo -n "Gathering OS info..."
mkdir $LOGDIR/OS
OS_FILES="/etc/redhat-release \
/etc/UnitedLinux-release \
/etc/SuSE-release \
/etc/asianux-release"
for file in $OS_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/OS
   fi
done

rpm -qa  > $LOGDIR/OS/rpm_list.out
uname -a > $LOGDIR/OS/uname

if test -d /sys
then
   ls -alRF /sys > $LOGDIR/OS/ls_sys.out
fi

#####################################################################
# Gather boot info
#####################################################################

/bin/echo
/bin/echo -n "Gathering boot info..."
mkdir $LOGDIR/boot
BOOT_FILES="/boot/grub/grub.conf \
/boot/grub/menu.lst \
/boot/efi/efi/SuSE/elilo.conf \
/boot/efi/efi/asianux/elilo.conf \
/boot/efi/efi/redhat/elilo.conf \
/etc/lilo.conf \
/etc/elilo.conf \
/etc/yaboot.conf"

for file in $BOOT_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/boot
   fi
done

if test -f /etc/grub.conf
then
   cp /etc/grub.conf $LOGDIR/boot/etc_grub.conf
fi

ls -alRF /boot > $LOGDIR/boot/ls_boot.out

#####################################################################
# Gather scli info
#####################################################################

/bin/echo
/bin/echo -n "Searching for SANsurfer CLI... "
if test -x /usr/local/bin/scli
then
   /bin/echo "detected"
   /bin/echo -n "Gathering HBA configuration information ..."
   scli -z all > $LOGDIR/scli.out 2>&1
   /bin/echo "done"
else
   /bin/echo "not installed"
fi

####################################################################
# Gathering the HBA information 
####################################################################

# grep with -m in grep we select one of the available tool

cd $CURRENTDIR
if [ -d "`ls | grep -m 1 ql-hba-snapshot`" ]; then
	SNAPSHOTDIR=`ls | grep -m 1 ql-hba-snapshot`	
	/bin/echo "Gathering HBA Information..."
	/bin/echo "SYSFS based" > $LOGDIR/hbainfo.out
	/bin/echo "" >> $LOGDIR/hbainfo.out
	sh ${SNAPSHOTDIR}/ql-hba-snapshot.sh -a >> $LOGDIR/hbainfo.out
	
	/bin/echo "" >> $LOGDIR/hbainfo.out
	/bin/echo "" >> $LOGDIR/hbainfo.out
	/bin/echo "PROC based" >> $LOGDIR/hbainfo.out
	/bin/echo "" >> $LOGDIR/hbainfo.out
	sh ${SNAPSHOTDIR}/ql-hba-snapshot.sh -proc -a >> $LOGDIR/hbainfo.out
	
	/bin/echo "" >> $LOGDIR/hbainfo.out
	/bin/echo "" >> $LOGDIR/hbainfo.out
	/bin/echo "Statistical Information" >> $LOGDIR/hbainfo.out
	/bin/echo "" >> $LOGDIR/hbainfo.out
	sh ${SNAPSHOTDIR}/ql-hba-snapshot.sh -s -a >> $LOGDIR/hbainfo.out
	
	/bin/echo "" >> $LOGDIR/hbainfo.out
	/bin/echo "" >> $LOGDIR/hbainfo.out
	/bin/echo "Parameter Information" >> $LOGDIR/hbainfo.out
	/bin/echo "" >> $LOGDIR/hbainfo.out
	sh ${SNAPSHOTDIR}/ql-hba-snapshot.sh -p >> $LOGDIR/hbainfo.out
	
fi
# Give message when multiple or no tools bundled

if [ `ls | grep -c ql-hba-snapshot` != "1" ]; then
		/bin/echo "HBA Snapshot tool not bundled correctly" >> $LOGDIR/hbainfo.out
fi

#####################################################################
# Create compressed archive of results Send mails and... then clean up
#####################################################################

/bin/echo "Creating compressed archive and cleaning up... "
((loop = 0))
cd /tmp
tar czf $LOGDIR.tgz ./QLogicDiag-$CurDate-$REVISION
if test $? -ne 0
	then
   	/bin/echo "*!*! Error while archiving the support data."
   	/bin/echo "     Please tar and compress $LOGDIR by hand"
   	/bin/echo "     and Email it to support@qlogic.com"
else
	
	/bin/echo ""
	/bin/echo "Following archive has been created \"/tmp/QLogicDiag-$CurDate-$REVISION.tgz\""
	/bin/echo "Please send this archive to appropriate QLogic contact."
	cd ${CURRENTDIR}
	which mutt >& /dev/null
	if test $? -eq  0
	then
	 while ((loop != QL_EXIT))
	 do
	/bin/echo -n "Do you wish to e-mail the compressed archive [y/n]?"
	read Option
	case $Option in 
			y | yes) 
	/bin/echo
	/bin/echo "Please Enter the e-Mail address of the recipient,"
	/bin/echo "or hit ENTER to select default ($QLOGICSUPPORT)"
	/bin/echo -n "e-Mail Address: "
	read emailAddress
		if test -z $emailAddress; then 
			emailAddress=$QLOGICSUPPORT
		fi
		/bin/echo
  		/bin/echo "Sending Mail to ${emailAddress}"
		((loop = 1))

	cat mail_body | sed -e "s/TARFILE/QLogic-${CurDate}-${REVISION}.tgz/" -e "s/NEW/${REVISION}/" | mutt -a "$LOGDIR.tgz" -s "System Information Data" "${emailAddress}";;

			n | no)
		((loop =1))
	/bin/echo "Please Email the file: $LOGDIR.tgz to support@qlogic.com";;
	esac
	done

fi
fi

#####################################################################
# All done ...
#####################################################################

exit

